<?PHP
/*
Plugin Name: On The Fly YouTube Embeds
Plugin URI: http://JoeAnzalone.com/plugins/on-the-fly-youtube-embeds
Description: Creates a page on your site that will play any YouTube video based on the requested URL without having to create a new page for each individual video.
Version: 1.0
Author: Joe Anzalone
Author URI: http://JoeAnzalone.com
License: GPL2
*/

/*
Thanks to Scott for helping me figure out how to create this fake page:
http://scott.sherrillmix.com/blog/blogger/creating-a-better-fake-post-with-a-wordpress-plugin/

Thanks to PressCoders for "Settings API Explained"
http://www.presscoders.com/2010/05/wordpress-settings-api-explained/
The code for the plugin options is based off of http://wordpress.org/extend/plugins/plugin-options-starter-kit/
*/

class on_the_fly_youtube_embeds {

	function get_video_info($vid){
		$vid_request_url = "https://gdata.youtube.com/feeds/api/videos/$vid?v=2&alt=jsonc";
		$remote_response = wp_remote_get($vid_request_url);
		$json = $remote_response['body'];
		$video_info = json_decode($json);
		// var_dump($video_info);
		return $video_info;
	}

	function get_embed_code($vid, $width, $height, $autoplay=0){
		$html = NULL;
		$html .= '<div class="embedded-youtube">';

		$html .= '<iframe src="http://www.youtube.com/embed/'.$vid.'?rel=0&autoplay=' . $autoplay . '" frameborder="0" width="'.$width.'" height="'.$height.'"></iframe>';

		$options = get_option('otfye_options');
		if( !empty($options['show_description']) && !empty($this->vid_info->data) ){
			$video_description = make_clickable( $this->vid_description );
			$html .= '<div class="video-description">' . $video_description . '</div>';
		}
		$html .= '</div>';
		return $html;
	}

	var $url_prefix = '';
	var $width = NULL;
	var $height = NULL;

    /**
     * The slug for the fake post.  This is the URL for your plugin, like:
     * http://site.com/about-me or http://site.com/?page_id=about-me
     * @var string
     */
    var $page_slug = '';
   
    /**
     * The title for your fake post.
     * @var string
     */
    var $page_title = '';

    /**
     * Allow pings?
     * @var string
     */
    var $ping_status = 'closed';
       
    /**
     * Class constructor
     */
    function __construct() {

    	$this->plugin_meta['name'] = 'On The Fly YouTube Embeds';
		// ------------------------------------------------------------------------
		// REGISTER HOOKS & CALLBACK FUNCTIONS:
		// ------------------------------------------------------------------------
		// HOOKS TO SETUP DEFAULT PLUGIN OPTIONS, HANDLE CLEAN-UP OF OPTIONS WHEN
		// PLUGIN IS DEACTIVATED AND DELETED, INITIALISE PLUGIN, ADD OPTIONS PAGE.
		// ------------------------------------------------------------------------

		add_action( 'admin_init', array(&$this, 'requires_wordpress_version' ) );
		register_activation_hook(dirname(__FILE__) . '/main.php', 'otfye_add_defaults');
		register_uninstall_hook(dirname(__FILE__) . '/main.php', 'otfye_delete_plugin_options');
		add_action('admin_init', array(&$this, 'admin_init_action') );
		add_action('admin_menu', array(&$this, 'admin_menu_action') );
		add_filter( 'plugin_action_links', array(&$this, 'plugin_action_links_filter'), 10, 2) ;

	    // $this->url_prefix = get_option( 'youtube_page_url_prefix', 'video' );
    	$options = get_option('otfye_options');
		$this->url_prefix = $options['slug'];

	    $this->width = $options['player']['width'];
	    $this->height = $options['player']['height'];

	    if( isset($options['player']['autoplay']) ){
	    	$this->autoplay = $options['player']['autoplay'];
	    } else {
	    	$this->autoplay = 0;
	    }
	    
        
        /**
         * We'll wait til WordPress has looked for posts, and then
         * check to see if the requested url matches our target.
         */
        add_filter('the_posts', array(&$this,'detect_post'));
        add_action('template_redirect',array(&$this,'template_redirect_action'));

        // add_action('admin_menu', array(&$this, 'register_settings_page'));
        // add_action('admin_menu', array(&$this, 'create_settings_page'));
		/* I think there's some bug with get_post_metadata. I just keep getting NULL!
        add_filter('get_post_metadata',array(&$this,'template_filter'), -100);
        */
    }

/* ****//* ****//* ****//* ****//* ****//* ****//* ****//* ****//* ****//* ****//* ****//* ****//* ****/
/*
http://stackoverflow.com/a/4975004/1027770
*/
function template_redirect_action() {
	if( !empty($this->vid)){
	    global $wp;
	    $plugindir = dirname( __FILE__ );

	    $options = get_option('otfye_options');
	    // $page_template = 'children.php';

	    if(!empty( $options['page_template'] )){
	    	$page_template = $options['page_template'];
		}

	    if (file_exists(TEMPLATEPATH . '/' . $page_template)) {
	        $return_template = TEMPLATEPATH . '/' . $page_template;
	        $this->do_theme_redirect($return_template);
	    }
    }
}

function do_theme_redirect($url) {
    global $post, $wp_query;
    if (have_posts()) {
        include($url);
        die();
    } else {
        $wp_query->is_404 = true;
    }
}
/* ****//* ****//* ****//* ****//* ****//* ****//* ****//* ****//* ****//* ****//* ****//* ****//* ****/

    /**
     * Called by the 'detect_post' action
     */
    function create_fake_post() {
        /**
         * What we are going to do here, is create a fake post.  A post
         * that doesn't actually exist. We're gonna fill it up with
         * whatever values you want.  The content of the post will be
         * the output from your plugin.
         */  
       
        /**
         * Create a fake post.
         */
        $post = new stdClass;
       
        /**
         * The author ID for the post.  Usually 1 is the sys admin.  Your
         * plugin can find out the real author ID without any trouble.
         */
        $post->post_author = 1;
       
        /**
         * The safe name for the post.  This is the post slug.
         */
        $post->post_name = $this->url_prefix . '/' . $this->vid;
       
        /**
         * Not sure if this is even important.  But gonna fill it up anyway.
         */
        $post->guid = get_bloginfo('wpurl') . '/' . $this->page_slug;
       
       
        /**
         * The title of the page.
         */
        $post->post_title = $this->page_title;
       
        /**
         * This is the content of the post.  This is where the output of
         * your plugin should go.  Just store the output from all your
         * plugin function calls, and put the output into this var.
         */
        $post->post_content = $this->get_embed_code($this->vid, $this->width, $this->height, $this->autoplay);
       
        /**
         * Fake post ID to prevent WP from trying to show comments for
         * a post that doesn't really exist.
         */
        $post->ID = -1;
       
        /**
         * Static means a page, not a post.
         */
        $post->post_status = 'static';
       
        /**
         * Turning off comments for the post.
         */
        $post->comment_status = 'closed';
       
        /**
         * Let people ping the post?  Probably doesn't matter since
         * comments are turned off, so not sure if WP would even
         * show the pings.
         */
        $post->ping_status = $this->ping_status;
       
        $post->comment_count = 0;
       
        $post->post_date = current_time('mysql');
        $post->post_date_gmt = current_time('mysql', 1);
 
        $post->post_type = 'page';
        $post->post_parent = 0;
        $post->post_mime_type = '';

        return($post);
    }
 
    function detect_post($posts){
        /**
         * Check if the requested page matches our target
         */
        global $wp;
        global $wp_query;
		$request = $wp->request; // video/9bZkp7q19f0
        

		preg_match("#{$this->url_prefix}/([a-zA-Z0-9_-]+)#", $request, $preg_matches);
		if(!empty($preg_matches[1])){
			$this->vid = $preg_matches[1];
		}

		if(!empty($_GET[$this->url_prefix])) {
			$this->vid = $_GET[$this->url_prefix];
        }

		if( !empty($this->vid) ) {
        	
			$this->vid_info = $this->get_video_info( $this->vid );
			$this->vid_description = esc_html( $this->vid_info->data->description );

			if(!empty($this->vid_info->data->title)){
				$this->page_title = $this->vid_info->data->title;
			}

			$wp->page ='';
			$wp->pagename = 'video';
			$wp->query_string = 'pagename=video';
			$wp->matched_query = 'pagename=video&page=';

            //Add the fake post
            $posts = NULL;
            $posts[] = $this->create_fake_post();
       
            /**
             * Trick wp_query into thinking this is a page (necessary for wp_title() at least)
             * Not sure if it's cheating or not to modify global variables in a filter
             * but it appears to work and the codex doesn't directly say not to.
             */
            $wp_query->is_page = true;
            //Not sure if this one is necessary but might as well set it like a true page
            
            // $wp_query->is_singular = true;
            $wp_query->is_single = false;

            $wp_query->is_home = false;
            $wp_query->is_archive = false;
            $wp_query->is_category = false;
            //Longer permalink structures may not match the fake post slug and cause a 404 error so we catch the error here
            unset($wp_query->query["error"]);
            $wp_query->query_vars["error"] = '';
            $wp_query->is_404 = false;
            
            $wp_query->is_attachment = false;
        }
        return $posts;
    }


	/* Settings Page Stuff */
	/*
	Why can't we just include a settings.php file or extend this class in a separate file?
	http://stackoverflow.com/questions/1957732/can-i-include-code-into-a-php-class
	*/
// ------------------------------------------------------------------------
// REQUIRE MINIMUM VERSION OF WORDPRESS:                                               
// ------------------------------------------------------------------------
// THIS IS USEFUL IF YOU REQUIRE A MINIMUM VERSION OF WORDPRESS TO RUN YOUR
// PLUGIN. IN THIS PLUGIN THE WP_EDITOR() FUNCTION REQUIRES WORDPRESS 3.3 
// OR ABOVE. ANYTHING LESS SHOWS A WARNING AND THE PLUGIN IS DEACTIVATED.                    
// ------------------------------------------------------------------------

function requires_wordpress_version() {
	global $wp_version;
	$plugin = plugin_basename( __FILE__ );
	$plugin_data = get_plugin_data( __FILE__, false );

	if ( version_compare($wp_version, "3.3", "<" ) ) {
		if( is_plugin_active($plugin) ) {
			deactivate_plugins( $plugin );
			wp_die( "'".$plugin_data['Name']."' requires WordPress 3.3 or higher, and has been deactivated! Please upgrade WordPress and try again.<br /><br />Back to <a href='".admin_url()."'>WordPress admin</a>." );
		}
	}
}

// Init plugin options to white list our options
function admin_init_action(){
	register_setting( 'otfye_plugin_options', 'otfye_options', array(&$this, 'validate_options') );
}

// Add menu page
function admin_menu_action() {
	add_options_page($this->plugin_meta['name'], 'OTF YouTube Embeds', 'manage_options', __FILE__, array(&$this, 'render_form'));
}

// ------------------------------------------------------------------------------
// CALLBACK FUNCTION SPECIFIED IN: add_options_page()
// ------------------------------------------------------------------------------
// THIS FUNCTION IS SPECIFIED IN add_options_page() AS THE CALLBACK FUNCTION THAT
// ACTUALLY RENDER THE PLUGIN OPTIONS FORM AS A SUB-MENU UNDER THE EXISTING
// SETTINGS ADMIN MENU.
// ------------------------------------------------------------------------------

// Render the Plugin options form
function render_form() {
	?>
	<div class="wrap">
		
		<!-- Display Plugin Icon, Header, and Description -->
		<div class="icon32" id="icon-options-general"><br></div>
		<h2><?PHP echo $this->plugin_meta['name']; ?></h2>
		<!-- <p>Change settings below</p> -->

		<!-- Beginning of the Plugin Options Form -->
		<form method="post" action="options.php">
			<?php settings_fields('otfye_plugin_options'); ?>
			<?php $options = get_option('otfye_options'); ?>

			<!-- Table Structure Containing Form Controls -->
			<!-- Each Plugin Option Defined on a New Table Row -->
			<table class="form-table">

				<!-- Textbox Control -->
				<tr>
					<th scope="row">Video page slug</th>
					<td>
						<?PHP

						$example_vids = array(  // Yes, this is *vital* to the operation of this plugin!
							'jOyFDvWf83w',
							'XvutrQoIAEM',
							'4PTsEjSEnhA',
							'_6-KspZegsE',
							'twqM56f_cVo',
							'bjWPyDMk8k8',
							'9bZkp7q19f0',
							'pHCdS7O248g',
							'AcusYrqBvYQ',
							'gqmo9-jVOcM',
							'SCsKRbChILA',
							'KQ6zr6kCPj8',
							'UtQbENQ4Zk0',
							'Yhx0vEecE5o',
						);
						$example_vid = $example_vids[array_rand($example_vids)];
						if( get_option('permalink_structure') ){
							$example_url = home_url() . '/<strong>' . $options['slug'] . "</strong>/$example_vid";
						} else {
							$example_url = home_url() . '/?<strong>' . $options['slug'] . "</strong>=$example_vid";
						}

						?>
						<input type="text" size="57" name="otfye_options[slug]" value="<?php echo $options['slug']; ?>" />
						<br /><span style="color:#666666;margin-left:2px;">Example: <?PHP echo $example_url; ?></span>
					</td>
				</tr>

				<tr>
					<th scope="row">Video Player Dimensions</th>
					<td>
						<label>Width<input type="text" size="5" name="otfye_options[player][width]" value="<?php echo $options['player']['width']; ?>" /></label>
						<label>Height<input type="text" size="5" name="otfye_options[player][height]" value="<?php echo $options['player']['height']; ?>" /></label>
					</td>
				</tr>

				<!-- Checkbox Buttons -->
				<tr valign="top">
					<th scope="row">Show video description?</th>
					<td>
						<!-- First checkbox button -->
						<label><input name="otfye_options[show_description]" type="checkbox" value="1" <?php if (isset($options['show_description'])) { checked('1', $options['show_description']); } ?> /> Display video description</label><br />
					</td>
				</tr>

				<!-- Checkbox Buttons -->
				<tr valign="top">
					<th scope="row">Autoplay?</th>
					<td>
						<!-- First checkbox button -->
						<label><input name="otfye_options[player][autoplay]" type="checkbox" value="1" <?php if (isset($options['player']['autoplay'])) { checked('1', $options['player']['autoplay']); } ?> /> Autoplay video</label><br />
					</td>
				</tr>

				<!-- Select Drop-Down Control -->
				<tr>
					<th scope="row">Page Template</th>
					<td>
						<select name='otfye_options[page_template]'>
							<?PHP
							$page_templates = array('Default Template' => 'default' ) + get_page_templates();
							 foreach ( $page_templates as $template_name => $template_file){
								echo '<option value="' . $template_file. '" '. selected($template_file, $options['page_template']) .'>' . $template_name . '</option>';

							} ?>
						</select>
						<span style="color:#666666;margin-left:2px;"><!-- Add a comment here to explain more about how to use the option above --></span>
					</td>
				</tr>

				<tr><td colspan="2"><div style="margin-top:10px;"></div></td></tr>
				<tr valign="top" style="border-top:#dddddd 1px solid;">
					<th scope="row">Database Options</th>
					<td>
						<label><input name="otfye_options[chk_default_options_db]" type="checkbox" value="1" <?php if (isset($options['chk_default_options_db'])) { checked('1', $options['chk_default_options_db']); } ?> /> Restore defaults upon plugin deactivation/reactivation</label>
						<br /><span style="color:#666666;margin-left:2px;">Only check this if you want to reset plugin settings upon Plugin reactivation</span>
					</td>
				</tr>
			</table>
			<p class="submit">
			<input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
			</p>
		</form>

	</div>
	<?php	
}

// Sanitize and validate input. Accepts an array, return a sanitized array.
function validate_options($input) {
	 // strip html from textboxes
	$input['textarea_one'] =  wp_filter_nohtml_kses($input['textarea_one']); // Sanitize textarea input (strip html tags, and escape characters)
	// $input['slug'] =  wp_filter_nohtml_kses($input['slug']); // Sanitize textbox input (strip html tags, and escape characters)
	$input['slug'] = sanitize_title($input['slug'], 'video');
	return $input;
}

// Display a Settings link on the main Plugins page
function plugin_action_links_filter( $links, $file ) {

	if ( $file == plugin_basename( __FILE__ ) ) {
		$otfye_links = '<a href="'.get_admin_url().'options-general.php?page=on-the-fly-youtube-embeds/main.php">'.__('Settings').'</a>';
		// make the 'Settings' link appear first
		array_unshift( $links, $otfye_links );
	}

	return $links;
}
/* /Settings Page Stuff */


}
 
/**
* Create an instance of our class.
*/
new on_the_fly_youtube_embeds;

// Delete options table entries ONLY when plugin deactivated AND deleted
function otfye_delete_plugin_options() {
	delete_option('otfye_options');
}

// ------------------------------------------------------------------------------
// CALLBACK FUNCTION FOR: register_activation_hook()
// ------------------------------------------------------------------------------
// THIS FUNCTION RUNS WHEN THE PLUGIN IS ACTIVATED. IF THERE ARE NO THEME OPTIONS
// CURRENTLY SET, OR THE USER HAS SELECTED THE CHECKBOX TO RESET OPTIONS TO THEIR
// DEFAULTS THEN THE OPTIONS ARE SET/RESET.
//
// OTHERWISE, THE PLUGIN OPTIONS REMAIN UNCHANGED.
// ------------------------------------------------------------------------------

// Define default option settings
function otfye_add_defaults() {
	$tmp = get_option('otfye_options');
    if(($tmp['chk_default_options_db']=='1')||(!is_array($tmp))) {
		delete_option('otfye_options'); // so we don't have to reset all the 'off' checkboxes too! (don't think this is needed but leave for now)
		$arr = array(	"show_description" => "1",
						"slug" => "video",
						"player" => array('width' => 850, 'height' => 480, 'autoplay' => "0"),
						"slug" => "video",
						"page_template" => "default",
						"chk_default_options_db" => "1",
		);
		update_option('otfye_options', $arr);
	}
}
?>